#pragma once
#ifndef HARDWARE_INFO_H
#define HARDWARE_INFO_H

#include "file/interface/hardware_info_structs.h"
#include "file/file_hardware_info.h"

class HardwareInfo
{
public:
    HardwareInfo();
    virtual ~HardwareInfo();

public:
    enum EnumDeviceType
    {
        PZEMasterIV,
        ZKMasterIV,
        BDEMasterIV,
        MobilMasterIV,
        MDEBoxIV,
        AEMasterIV,
        TimeboyIV,
        ExklusivLineIV,
        FlexMasterIV,
        TimeboyMobilPZE,
            // Bis hier waren die Nummern identisch mit der HardwareInfo-Vorgängerversion.
        IPCExtensionA, // Alte HW-Info nimmt Nummer fuer: EVOLine28
        EVOLine28,     // Alte HW-Info nimmt Nummer fuer: EVOLine43
        EVOLine43,     // Alte HW-Info nimmt Nummer fuer: IPCExtensionA
        IPCExtensionB,
        IPCExtensionC,
        ZKBox,         // Alte HW-Info nimmt Nummer fuer: IOBox
        IOBox,         // Alte HW-Info nimmt Nummer fuer: ZKBox
        MobilBox,
        TimeboyTBDock,
        MobileTBDock,
        ZKKnoten,
        EVO35Universal,
        SAMSOCLReader,
        EVO35,
        ZKReaderPlus,
        ZKZentrale,
    };
public:
    void reset();

    bool importStructHardwareInfo( const StructHardwareInfo &info );
    StructHardwareInfo exportStructHardwareInfo();
    StructHardwareInfo importedStructHardwareInfo();

    void setFileHardwareInfo( const FileHardwareInfo &file );
    FileHardwareInfo &fileHardwareInfo();

public:
    bool isMasterIV();
    bool isMasterIV( const StructHardwareInfo &info );


public:
    enum EnumHardwareConfigType
    {
        Mainboard,
        Module,
        ModuleWithPlaces,
        SubModule
    };

public:
    void setVersion( unsigned char version );
    unsigned char version();
    std::string deviceName(EnumDeviceType type);
    EnumDeviceType deviceType();
    EnumDeviceType deviceTypeOld();
    unsigned char mainBoardRevision();

public:
    listItemBoard collectModuleIds( const setModuleId &ids );
    // Shorties
    listItemBoard moduleRS232Communication( const setModuleRS232Communication &module = moduleRS232Communication_all );
    listItemBoard moduleRS485Communication( const setModuleRS485Communication &module = moduleRS485Communication_all );
    listItemBoard moduleUSBCommunication( const setModuleUSBCommunication &module = moduleUSBCommunication_all );
    listItemBoard moduleLANCommunication( const setModuleLANCommunication &module = moduleLANCommunication_all );
    listItemBoard moduleWLANCommunication( const setModuleWLANCommunication &module = moduleWLANCommunication_all );
    listItemBoard moduleMobileCommunication( const setModuleMobileCommunication &module = moduleMobileCommunication_all );
    listItemBoard moduleBluetoothCommunication( const setModuleBluetoothCommunication &module = moduleBluetoothCommunication_all );
    listItemBoard moduleMMCCommunication( const setModuleMMCCommunication &module = moduleMMCCommunication_all );
    listItemBoard moduleUSBHostCommunication( const setModuleUSBHostCommunication &module = moduleUSBHostCommunication_all );

    listItemBoard moduleAccessControl( const setModuleAccessControl &module = moduleAccessControl_all );
    listItemBoard moduleBiokey( const setModuleBiokey &module = moduleBiokey_all );
    listItemBoard moduleGPS( const setModuleGPS &module = moduleGPS_all );

    listItemBoard moduleDisplay( const setModuleDisplay &module = moduleDisplay_all );
    listItemBoard moduleTouch( const setModuleTouch &module = moduleTouch_all );

private:
    unsigned char version_;
    FileHardwareInfo _fileHardwareInfo;
    // Uebergangsweise beim Import die zugrundeliegende Struktur merken.
    StructHardwareInfo hwInfo_;
};

#endif
