#include "common/device_info.h"
#include "datafox/communication/module_hardware.h"

DeviceInfo::DeviceInfo()
{
    clear();
}

DeviceInfo::~DeviceInfo()
{
}

void DeviceInfo::clear()
{
    _versionBootloader = "";
    _bootloaderMode = false;
    _versionFirmware = "";

    _serialNumber = 0;

    _hardwareInfo.reset();

    _versionFingerprint = "";
    _versionUSBHost = "";
    _versionGPS = "";

    _macAddress = "";

    _softwareInfo.clear();
    _passwordKey = "";

    _firmwareOptions.clear();

    _accessControlMasterId = "";
    _timeboyGroupId = "";
}

void DeviceInfo::setVersionBootloader( const std::string &version )
{
    _versionBootloader = version;
}

std::string DeviceInfo::versionBootloader() const
{
    return _versionBootloader;
}

void DeviceInfo::setBootloaderMode( bool active )
{
    _bootloaderMode = active;
}

bool DeviceInfo::bootloaderMode() const
{
    return _bootloaderMode;
}

void DeviceInfo::setHardwareInfo( const HardwareInfo &hardwareInfo )
{
    _hardwareInfo = hardwareInfo;
}

HardwareInfo &DeviceInfo::hardwareInfo()
{
    return _hardwareInfo;
}

void DeviceInfo::setFileHardwareInfo( const FileHardwareInfo &file )
{
    _hardwareInfo.setFileHardwareInfo( file );
}

FileHardwareInfo &DeviceInfo::fileHardwareInfo()
{
    return _hardwareInfo.fileHardwareInfo();
}

listItemBoard DeviceInfo::collectModuleIds( const setModuleId &ids )
{
    return _hardwareInfo.collectModuleIds( ids );
}

void DeviceInfo::setVersionFirmware( const std::string &version )
{
    _versionFirmware = version;
}

std::string DeviceInfo::versionFirmware() const
{
    return _versionFirmware;
}

void DeviceInfo::setSoftwareInfo( const SoftwareInfo &softwareInfo )
{
    _softwareInfo = softwareInfo;
}

SoftwareInfo &DeviceInfo::softwareInfo()
{
    return _softwareInfo;
}

void DeviceInfo::setFirmwareOptions( const FirmwareOptions &firmwareOptions )
{
    _firmwareOptions = firmwareOptions;
}

FirmwareOptions &DeviceInfo::firmwareOptions()
{
    return _firmwareOptions;
}

void DeviceInfo::setSerialNumber( unsigned int serial )
{
    _serialNumber = serial;
}

unsigned int DeviceInfo::serialNumber() const
{
    return _serialNumber;
}

void DeviceInfo::setPasswordKey( const std::string &key )
{
    _passwordKey = key;
}

std::string DeviceInfo::passwordKey() const
{
    return _passwordKey;
}

void DeviceInfo::setVersionFingerprint( const std::string &version )
{
    _versionFingerprint = version;
}

std::string DeviceInfo::versionFingerprint() const
{
    return _versionFingerprint;
}

void DeviceInfo::setVersionUSBHost( const std::string &version )
{
    _versionUSBHost = version;
}

std::string DeviceInfo::versionUSBHost() const
{
    return _versionUSBHost;
}

void DeviceInfo::setVersionGPS( const std::string &version )
{
    _versionGPS = version;
}

std::string DeviceInfo::versionGPS() const
{
    return _versionGPS;
}

void DeviceInfo::setVersionTCPIP( const std::string &version )
{
    _versionTCPIP = version;
}

std::string DeviceInfo::versionTCPIP() const
{
    return _versionTCPIP;
}

void DeviceInfo::setMacAddress( const std::string &address )
{
    _macAddress = address;
}

std::string DeviceInfo::macAddress() const
{
    return _macAddress;
}

void DeviceInfo::setAccessControlMasterId( const std::string &id )
{
    _accessControlMasterId = id;
}

std::string DeviceInfo::accessControlMasterId() const
{
    return _accessControlMasterId;
}

void DeviceInfo::setTimeboyGroupId( const std::string &id )
{
    _timeboyGroupId = id;
}

std::string DeviceInfo::timeboyGroupId() const
{
    return _timeboyGroupId;
}

void DeviceInfo::setBoardApplication( const std::string &val )
{
    _boardApplication = val;
}

std::string DeviceInfo::boardApplication() const
{
    return _boardApplication;
}

void DeviceInfo::setBoardPlace_0_Output_1_Mode( const std::string &mode )
{
    _boardPlace_0_Output_1_Mode = mode;
}

std::string DeviceInfo::boardPlace_0_Output_1_Mode() const
{
    return _boardPlace_0_Output_1_Mode;
}

void DeviceInfo::setBoardPlace_0_Serial_1_Use( const std::string &use )
{
    _boardPlace_0_Serial_1_Use = use;
}

std::string DeviceInfo::boardPlace_0_Serial_1_Use() const
{
    return _boardPlace_0_Serial_1_Use;
}

void DeviceInfo::setBoardPlace_0_Serial_2_Use( const std::string &use )
{
    _boardPlace_0_Serial_2_Use = use;
}

std::string DeviceInfo::boardPlace_0_Serial_2_Use() const
{
    return _boardPlace_0_Serial_2_Use;
}

void DeviceInfo::setTimeStartOS( const std::string &val )
{
    _timeStartOS = val;
}

std::string DeviceInfo::timeStartOS() const
{
    return _timeStartOS;
}

void DeviceInfo::setOffTimeLong( const std::string &val )
{
    _offTimeLong = val;
}

std::string DeviceInfo::offTimeLong() const
{
    return _offTimeLong;
}

void DeviceInfo::setOffTimeShort( const std::string &val )
{
    _offTimeShort = val;
}

std::string DeviceInfo::offTimeShort() const
{
    return _offTimeShort;
}

void DeviceInfo::setOnTime( const std::string &val )
{
    _onTime = val;
}

std::string DeviceInfo::onTime() const
{
    return _onTime;
}

bool DeviceInfo::importStructHardwareInfo( const StructHardwareInfo &info )
{  
    return _hardwareInfo.importStructHardwareInfo( info );
}

StructHardwareInfo DeviceInfo::structHardwareInfo()
{
    return _hardwareInfo.exportStructHardwareInfo();
}

void DeviceInfo::setStructSoftwareInfo( const StructSoftwareInfo &info )
{
    _softwareInfo.setStructSoftwareInfo( info );
}

StructSoftwareInfo DeviceInfo::structSoftwareInfo() const
{
    return _softwareInfo.structSoftwareInfo();
}

StructSoftwareInfo DeviceInfo::checkInfo( const StructSoftwareInfo &info )
{
    return _softwareInfo.check( info );
}

int DeviceInfo::SoftwareInfoToString( const StructSoftwareInfo &swInfo, std::string &info )
{
    return _softwareInfo.toString( swInfo, info );
}

int DeviceInfo::compareInfos( const StructHardwareInfo &hwInfo, const StructSoftwareInfo &swInfo )
{
    unsigned char type;
    unsigned int bit;
    int res = 1;

    // Keine Hardwareinfo per Software gesetzt worden?
    if ( hwInfo.version == 0 )
    { 
        // Dieses Betrifft lediglich die Geraete: PZE-/ZK-MasterIV

        // Je nach Geraetetyp Standardkonfiguration annehmen.
        // Die meisten Optionen werden durch die (04.00.23) Firmware ermittelt und gesetzt.
        switch ( hwInfo.typeDevice )
        {
        case 0: //IV_VALUE_TYPE_DEVICE_PZE
            // Zusaetzlich Tastatur und Display sicherstellen
            const_cast<StructHardwareInfo &>( hwInfo ).typeDisplay = 1;//IV_VALUE_TYPE_DISPLAY_SMALL;
            const_cast<StructHardwareInfo &>( hwInfo ).typeKeypad  = 1;//IV_VALUE_TYPE_KEYPAD_9;
            break;
        case 1: //IV_VALUE_TYPE_DEVICE_ZK
            // Zusaetzlich Zutrittskontrolle sicherstellen
            const_cast<StructHardwareInfo &>( hwInfo ).componentRS485_1 = 1;
            const_cast<StructHardwareInfo &>( hwInfo ).typeExternalReader = 1;//IV_VALUE_TYPE_EXTERNAL_READER_GIS;
            break;
        }

        // Ist die ZK-Option in Verwendung?
        if ( hwInfo.administrationFlags & HIAF_HAS_ZK )
        {  
            // ZK-Option mit GIS Lesern.
            const_cast<StructHardwareInfo &>( hwInfo ).componentRS485_1 = 1;
            const_cast<StructHardwareInfo &>( hwInfo ).typeExternalReader = 1;
        }
    }

    // Geraetetyp pruefen
    if ( ((1 << hwInfo.typeDevice) & swInfo.deviceExtended) == 0 )
    {
        m_errorText = "Der Geraetetyp wird nicht unterstuetzt.";
        return 0;
    }

    type = 0;
    // Boardtyp pruefen
    // Bei der Switch müssen sich die case Fälle nach dem Hinzufügen der Version richten, da type dann die Bitposition der Softwareinfo bestimmt.
    // Der Boardtyp mussauch in der Datei Commmon/softwareinfo.h als Define angelegt werden!!!!!!!!!!
    switch ( hwInfo.typeBoard )
    {
    case IV_VALUE_TYPE_BOARD_4_8: type++;
    case IV_VALUE_TYPE_BOARD_2_7: type++;
    case IV_VALUE_TYPE_BOARD_1_8: type++;
	case IV_VALUE_TYPE_BOARD_4_7: type++;
	case IV_VALUE_TYPE_BOARD_4_6: type++;
    case IV_VALUE_TYPE_BOARD_2_6: type++;
    case IV_VALUE_TYPE_BOARD_1_7: type++;
    case IV_VALUE_TYPE_BOARD_2_5: type++;
    case IV_VALUE_TYPE_BOARD_2_2: type++;
    case IV_VALUE_TYPE_BOARD_1_6: type++;
    case IV_VALUE_TYPE_BOARD_4_5: type++;
    case IV_VALUE_TYPE_BOARD_1_5: type++;
    case IV_VALUE_TYPE_BOARD_4_4: type++;
    case IV_VALUE_TYPE_BOARD_3_1: type++;
    case IV_VALUE_TYPE_BOARD_2_4: type++;
    case IV_VALUE_TYPE_BOARD_2_3: type++;
    case IV_VALUE_TYPE_BOARD_4_3: type++;
    case IV_VALUE_TYPE_BOARD_3_4: type++;
    case IV_VALUE_TYPE_BOARD_4_2: type++;
    case IV_VALUE_TYPE_BOARD_4_1: type++;
    case IV_VALUE_TYPE_BOARD_4_0: type++;
    case IV_VALUE_TYPE_BOARD_3_0: type++; // 10000000 0x80
    case IV_VALUE_TYPE_BOARD_2_1: type++; //  1000000 0x40
    case IV_VALUE_TYPE_BOARD_2_0: type++; //   100000 0x20
    case IV_VALUE_TYPE_BOARD_1_4: type++; //    10000 0x10
    case IV_VALUE_TYPE_BOARD_1_3: type++; //     1000 0x08
    case IV_VALUE_TYPE_BOARD_1_2: type++; //      100 0x04
    case IV_VALUE_TYPE_BOARD_1_1: type++; //       10 0x02
    case IV_VALUE_TYPE_BOARD_1_0:         //        1 0x01
        break;

	default:
        m_errorText = "Die Boardrevision wird nicht unterstuetzt.";
        return 0;
    }

    if ( ((1 << type) & swInfo.boardExtended) == 0 )
    {
        m_errorText = "Die Boardrevision wird nicht unterstuetzt.";
        return 0;
    }

    // Handelt es sich nicht um einen MasterIV.
    if ( !_hardwareInfo.isMasterIV( hwInfo ) )
        return res;

    // IO-Modul pruefen
    if ( hwInfo.optionIOModule && !swInfo.iomodule )
    {
        // IOModule bei folgenden Geraeten nicht moeglich!
        // Die gewaehlte Option ist unnoetig.
        if ( hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_PZE && 
            hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_TIMEBOY && 
            hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE )
        {
            m_errorText = "Das IO-Modul wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Timeboyanbindung pruefen
    if ( hwInfo.optionTBAdaption && !swInfo.docking )
    {
        // Anbindung Timeboy bei folgenden Geraeten nicht moeglich!
        // Die gewaehlte Option ist unnoetig.
        if ( hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_BDE &&
            hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_TIMEBOY &&
            hwInfo.typeDevice != IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE )
        {
            m_errorText = "Die Timeboyanbindung wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Comserver pruefen
    if ( hwInfo.optionComserver && !swInfo.comserver )
    {
        m_errorText = "Der Comserver-Modus wird nicht unterstuetzt.";
        return 0;
    }

    // Transponderwert schreiben pruefen
    if ( hwInfo.optionTPWrite && !swInfo.transponder_write )
    {
        // Wenn kein Transponder vorliegt, ist die Option unnoetig!
        // Auch wenn kein Display vorliegt!
        if ( hwInfo.typeTransponder != IV_VALUE_TYPE_TRANSPONDER_NONE && hwInfo.typeDisplay != IV_VALUE_TYPE_DISPLAY_NONE )
        {
            m_errorText = "Das Schreiben eines Transponders wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Http-Protokoll pruefen
    if ( hwInfo.optionHttp && !swInfo.http )
    {
        // Wenn kein Modem eingebaut, ist die Option unnoetig!
        if ( hwInfo.typeModemMC != IV_VALUE_TYPE_MODEM_MC_NONE )
        {
            m_errorText = "Das Http-Protokoll wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Sprachdatei pruefen
    if ( hwInfo.optionLanguage && !swInfo.language )
    {
        m_errorText = "Die Sprachdateien werden nicht unterstuetzt.";
        return 0;
    }

    // Update des Biokey pruefen
    if ( hwInfo.optionBiokeyUpdate && !swInfo.biokey_update )
    {
        // Wenn kein Biokey-Modul eingebaut, ist die Option unnoetig!
        if ( hwInfo.typeFingerprint != IV_VALUE_TYPE_FINGERPRINT_NONE )
        {
            m_errorText = "Das Update des Biokey wird nicht unterstuetzt.";
            return 0;
        }
    }

    // RS485 pruefen
    if ( hwInfo.componentRS485_0 && !swInfo.rs485 )
    {
        m_errorText = "Die RS485-Kommunikation wird nicht unterstuetzt.";
        return 0;
    }

    // TCPIP pruefen
    if ( hwInfo.typeTCPIP && (((1 << (hwInfo.typeTCPIP - 1)) & swInfo.tcpip)) == 0 )
    {
        m_errorText = "Die TCPIP-Kommunikation wird nicht unterstuetzt.";
        return 0;
    }

    // ModemMCxx pruefen
    if ( hwInfo.typeModemMC && (((1 << (hwInfo.typeModemMC - 1)) & swInfo.modem)) == 0 )
    {
        m_errorText = "Das verwendete Modem wird nicht unterstuetzt.";
        return 0;
    }

    // RS485 (intern) ermitteln
    if ( hwInfo.componentRS485_1 && !swInfo.access )
    {
        m_errorText = "Die Zutrittskontrolle wird nicht unterstuetzt.";
        return 0;
    }

    // Display pruefen
    if ( hwInfo.typeDisplay && ((1 << (hwInfo.typeDisplay - 1)) & swInfo.display) == 0 )
    {
        m_errorText = "Das verwendete Display wird nicht unterstuetzt.";
        return 0;
    }

    // Tastatur pruefen
    if ( hwInfo.typeKeypad && ((1 << (hwInfo.typeKeypad - 1)) & swInfo.keypad) == 0 )
    {
        m_errorText = "Die verwendete Tastatur wird nicht unterstuetzt.";
        return 0;
    }

    // Transponder pruefen
    if ( hwInfo.typeTransponder && ((1 << (hwInfo.typeTransponder - 1)) & swInfo.transponderExtended) == 0 )
    {
        m_errorText = "Der verwendete Transponderleser wird nicht unterstuetzt.";
        return 0;
    }

    // MMC pruefen
    if ( hwInfo.typeMMC && ((1 << (hwInfo.typeMMC - 1)) & swInfo.mmc) == 0 )
    {
        m_errorText = "Die verwendete MMC/USB-Host-Option wird nicht unterstuetzt.";
        return 0;
    }

    // Funk pruefen
    if ( hwInfo.typeRadio && ((1 << (hwInfo.typeRadio - 1)) & swInfo.radio) == 0 )
    {
        m_errorText = "Das verwendete Funkmodul wird nicht unterstuetzt.";
        return 0;
    }

    // GPS pruefen
    if ( hwInfo.typeGPS && ((1 << (hwInfo.typeGPS - 1)) & swInfo.gps) == 0 )
    {
        m_errorText = "Das verwendete GPS-Modul wird nicht unterstuetzt.";
        return 0;
    }

    // USB pruefen
    if ( hwInfo.componentUSB && !swInfo.usb )
    {
        m_errorText = "Die USB-Kommunikation wird nicht unterstuetzt.";
        return 0;
    }

    // DCF77 / Barcode (intern) pruefen
    if ( hwInfo.typeDCF77 && ((1 << (hwInfo.typeDCF77 - 1)) & swInfo.dcf77) == 0 )
    {
        m_errorText = "Das verwendete DCF77/Barcode (intern) - Modul wird nicht unterstuetzt.";
        return 0;
    }


    // DigitalIn pruefen
    if ( hwInfo.componentInAnalog )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( (bit & hwInfo.componentInAnalog) && !(bit & swInfo.analog_in) )
            {
                m_errorText = "Analoger Eingang wird nicht unterstuetzt.";
                return 0;
            }

            if ( bit == 0x8000000 )
                break;
        }
    }

    // DigitalIn pruefen
    if ( hwInfo.componentInDigital )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( (bit & hwInfo.componentInDigital) && !(bit & swInfo.digital_in) ) 
            {
                m_errorText = "Digitaler Eingang wird nicht unterstuetzt.";
                return 0;
            }

            if ( bit == 0x8000000 )
                break;
        }
    }

    // DigitalOut pruefen
    if ( hwInfo.componentOutDigital )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( (bit & hwInfo.componentOutDigital) && !(bit & swInfo.digital_out) )
            {
                m_errorText = "Digitaler Ausgang wird nicht unterstuetzt.";
                return 0;
            }

            if ( bit == 0x8000000 )
                break;
        }
    }

    // Fingerprint pruefen
    if ( hwInfo.typeFingerprint && ((1 << (hwInfo.typeFingerprint - 1)) & swInfo.fingerprint) == 0 )
    {
        m_errorText = "Das verwendete Fingerprint-Modul wird nicht unterstuetzt.";
        return 0;
    }

    // Externe Zutrittsleser pruefen
    if ( hwInfo.typeExternalReader && ((hwInfo.typeExternalReader & swInfo.access) == 0) )
    {
        m_errorText = "Die Externen Zutrittsleser werden nicht unterstuetzt.";
        return 0;
    }

    // Sollen die Setupoptionen geprueft werden
    if ( hwInfo.administrationFlags & HIAF_HAS_OPTIONS3 )
    {
        // SetupOption pruefen
        if ( hwInfo.optionsSetup )
        {
            for ( bit = 1; ; bit <<= 1 )
            {
                if ( (bit & hwInfo.optionsSetup) && !(bit & swInfo.setup_options) )
                {
                    m_errorText = "Softwareoption wird nicht unterstuetzt.";
                    return 0;
                }

                if ( bit == 0x8000 )
                    break;
            }
        }

    }

    // Sind die Optionen der Gruppe 4 zum Pruefen vorhanden?
    if ( hwInfo.administrationFlags & HIAF_HAS_OPTIONS4 )
    {

        // Touch pruefen
        if ( hwInfo.typeTouch && ((1 << (hwInfo.typeTouch - 1)) & swInfo.touch) == 0 )
        {
            m_errorText = "Das verwendete Touchpanel wird nicht unterstuetzt.";
            return 0;
        }

        // Sensor pruefen
        if ( hwInfo.typeSensor && ((1 << (hwInfo.typeSensor - 1)) & swInfo.sensor) == 0 )
        {
            m_errorText = "Der verwendete Sensor wird nicht unterstuetzt.";
            return 0;
        }

        // Batterie pruefen
        if ( hwInfo.componentBattery && !swInfo.battery )
        {
            m_errorText = "Die benoetigte Batterieueberwachung wird nicht unterstuetzt.";
            return 0;
        }

        // Ladeschaltung pruefen
        if ( hwInfo.componentCharge && !swInfo.charge )
        {
            m_errorText = "Die Akkuueberwachung (Ladeschaltung) wird nicht unterstuetzt.";
            return 0;
        }
    }

    // Sind die Optionen der Gruppe 5 zum Pruefen vorhanden?
    if ( hwInfo.administrationFlags & HIAF_HAS_OPTIONS5 )
    {
        // Erweiterte Optionen pruefen
        if ( hwInfo.optionsPayable )
        {
            for ( bit = 1; ; bit <<= 1 )
            {
                if ( (bit & hwInfo.optionsPayable) && !(bit & swInfo.payable_options) )
                {
                    m_errorText = "Erweiterte Option wird nicht unterstuetzt.";
                    return 0;
                }

                if ( bit == 0x80 )
                    break;
            }
        }
    }

    return res;
}

void DeviceInfo::setSystemFlags( unsigned int flags )
{
    _firmwareOptions.setSystemFlags( flags );
}

unsigned int DeviceInfo::systemFlags() const
{
    return _firmwareOptions.getSystemFlags();
}

void DeviceInfo::setTypeExternalReader( unsigned char type )
{
    _firmwareOptions.setTypeExternalReader( type );
}

unsigned char DeviceInfo::getTypeExternalReader()
{
    return _firmwareOptions.getTypeExternalReader();
}

void DeviceInfo::setOptionIOModule( unsigned char enabled )
{
    _firmwareOptions.setOptionIOModule( enabled );
}

unsigned char DeviceInfo::getOptionIOModule()
{
    return _firmwareOptions.getOptionIOModule();
}

void DeviceInfo::setOptionTBAdaption( unsigned char enabled )
{
    _firmwareOptions.setOptionTBAdaption( enabled );
}

unsigned char DeviceInfo::getOptionTBAdaption()
{
    return _firmwareOptions.getOptionTBAdaption();
}

void DeviceInfo::setOptionComServer( unsigned char enabled )
{
    _firmwareOptions.setOptionComServer( enabled );
}

unsigned char DeviceInfo::getOptionComServer()
{
    return _firmwareOptions.getOptionComServer();
}

void DeviceInfo::setOptionTPWrite( unsigned char enabled )
{
    _firmwareOptions.setOptionTPWrite( enabled );
}

unsigned char DeviceInfo::getOptionTPWrite()
{
    return _firmwareOptions.getOptionTPWrite();
}

void DeviceInfo::setOptionHTTP( unsigned char enabled )
{
    _firmwareOptions.setOptionHTTP( enabled );
}

unsigned char DeviceInfo::getOptionHTTP()
{
    return _firmwareOptions.getOptionHTTP();
}

void DeviceInfo::setOptionLanguage( unsigned char enabled )
{
    _firmwareOptions.setOptionLanguage( enabled );
}

unsigned char DeviceInfo::getOptionLanguage()
{
    return _firmwareOptions.getOptionLanguage();
}

void DeviceInfo::setOptionBiokeyUpdate( unsigned char enabled )
{
    _firmwareOptions.setOptionBiokeyUpdate( enabled );
}

unsigned char DeviceInfo::getOptionBiokeyUpdate()
{
    return _firmwareOptions.getOptionBiokeyUpdate();
}

void DeviceInfo::setSetupOptions( unsigned short mask, unsigned short options )
{
    _firmwareOptions.setSetupOptions( mask, options );
}

unsigned short DeviceInfo::getSetupOptions( unsigned short mask )
{
    return _firmwareOptions.getSetupOptions( mask );
}

void DeviceInfo::setPayableOptions( unsigned char mask, unsigned char options )
{
    _firmwareOptions.setPayableOptions( mask, options );
}

bool DeviceInfo::isPayableOption( FirmwareOptions::EnumPayableOption option )
{
    return _firmwareOptions.isPayableOption( option );
}

